/**
IBM SOFTWARE DISCLAIMER 

Java array package (draft 0.2). Copyright (1998), International Business 
Machines Corporation.

Permission to use, copy, modify and distribute this software for any
noncommercial purpose and without fee is hereby granted, provided that
this copyright and permission notice appear on all copies of the
software. The name of the IBM Corporation may not be used in any
advertising or publicity pertaining to the use of the software. IBM
makes no warranty or representations about the suitability of the
software for any purpose.  It is provided "AS IS" without any express
or implied warranty, including the implied warranties of
merchantability, fitness for a particular purpose and non-infringement.
IBM shall not be liable for any direct, indirect, special or
consequential damages resulting from the loss of use, data or projects,
whether in an action of contract or tort, arising out of or in
connection with the use or performance of this software.
*/

package array;

/**
   array = Java array package
   <P>
   The Java array package provides the basic functionality of 
   Fortran 90 arrays in Java. 
   Arrays are multidimensional, rectangular grids of elements. 
   The array class hierarchy is as follows:
   <P>
   Array ----> &lt;type&gt;Array ----> &lt;type&gt;Array&lt;rank&gt;
   <P>
   where Array is an interface, &lt;type&gt;Array is an abstract 
   class and &lt;type&gt;Array&lt;rank&gt; is a (concrete) class. 
   Only objects of class &lt;type&gt;Array&lt;rank&gt; can be 
   instantiated. 
   The following &lt;type&gt;s are valid:
   <P>
   boolean, byte, short, int, long, char, float, double,
   FloatComplex, DoubleComplex, and Object.
   <P>
   The following &lt;rank&gt;s are valid:
   <P>
   0D (for 0-dimensional arrays), 1D (for 1-dimensional arrays), 
   2D, 3D, 4D, 5D, 6D, 7D.
   <P>
   Note: This distribution only contains the classes for 
   &lt;type&gt;=double and &lt;rank&gt;=0D,1D,2D,3D.
   <P>
   The Array interface is implemented by all abstract array classes
   of the form &lt;type&gt;Array. It declares the basic methods that are
   common to all arrays, independent of &lt;type&gt; and &lt;rank&gt;
*/

public interface Array {

    /**
       Return the rank of the array.
    */
    public int rank();

    /**
       Return the number of elements in the array.
    */
    public int size();

    /**
       Return the extent of the array along its i-th axis.
       @param i	array axis (0 <= i < rank())
       @exception InvalidArrayAxisException i must be between 0 and rank-1
    */
    public int size(int i) throws InvalidArrayAxisException;

    /**
       Return the shape of the array as an int[] I of length rank().
       I(i) = size(i)
    */
    public int[] shape();

    /**
       Return an Object which has the same value as the 
       specified element of the array.
       @param index	index[i] = index along i-th axis, 0 <= index[i] < size(i)
       @exception InvalidArrayIndexException index[] must be of length equal to rank
    */
    public Object getAsObject(int[] index) 
	throws InvalidArrayIndexException, ArrayIndexOutOfBoundsException;

    /**
       Sets the specified element of the array to a particular value.
       @param index	index[i] = index along i-th axis, 0 <= index[i] < size(i)
       @param obj	the array element is set to the value of this Object
       @exception InvalidArrayIndexException index[] must be of length equal to rank
    */
    public void setToObject(int[] index, Object obj) 
	throws InvalidArrayIndexException, ArrayIndexOutOfBoundsException,
	NullPointerException;
}
    
